var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var path = require('path');
var child_process = require('child_process');
var PLUGIN_NAME = 'memory-usage';
var PLUGIN_FILE = 'memory-footprint.jar';
var MEMORY_FOOT_PRINT_TOOL_PATH = path.join(wfsUtil.getSystemPath('userData'), 'plugins', PLUGIN_NAME, PLUGIN_FILE);
//WO-P8
//https://developer.android.com/docs/quality-guidelines/wear-app-quality#performance-functionality
//https://developer.android.com/training/wearables/wff/memory-usage
var ACTIVE_MEMORY_LIMIT = 100;
var AMBIENT_MEMORY_LIMIT = 10;
var NORMAL_MEMORY_LIMIT = ACTIVE_MEMORY_LIMIT * 1024 * 1024; //100MB
var AOD_MEMORY_LIMIT = AMBIENT_MEMORY_LIMIT * 1024 * 1024; //10MB
var NORMAL_MEMORY_LIMIT_SIZE = bytesToSize(NORMAL_MEMORY_LIMIT);
var AOD_MEMORY_LIMIT_SIZE = bytesToSize(AOD_MEMORY_LIMIT);
//https://github.com/google/watchface/blob/e14415090090cc9ecd4930afbdd911da4f698003/play-validations/memory-footprint/src/main/java/com/google/wear/watchface/dfx/memory/MemoryFootprint.java#L92C2-L100C1
function bytesToSize(bytes) {
    var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
    if (bytes === 0)
        return { size: bytes, unit: 'KB' };
    var i = Math.floor(Math.log(Math.abs(bytes)) / Math.log(1024));
    if (i === 0)
        return { size: bytes, unit: sizes[i] };
    return { size: Math.ceil(bytes / Math.pow(1024, i)), unit: sizes[i] };
}
function memoryFootprintResultDecorator(result) {
    var _a, _b;
    var resultJson = JSON.parse(result);
    if (!resultJson)
        return undefined;
    var normal = (_a = resultJson.maxActiveBytes) !== null && _a !== void 0 ? _a : 0;
    var memoryInNormal = bytesToSize(normal);
    var aod = (_b = resultJson.maxAmbientBytes) !== null && _b !== void 0 ? _b : 0;
    var memoryInAod = bytesToSize(aod);
    var memoryResults = [
        {
            type: 'Normal mode',
            currentMemory: "".concat(memoryInNormal.size.toFixed(1), " ").concat(memoryInNormal.unit),
            limit: "(Max ".concat(NORMAL_MEMORY_LIMIT_SIZE.size, " ").concat(NORMAL_MEMORY_LIMIT_SIZE.unit, ")"),
            hasExceeded: normal > NORMAL_MEMORY_LIMIT,
        },
        {
            type: 'Always-on mode',
            currentMemory: "".concat(memoryInAod.size.toFixed(1), " ").concat(memoryInAod.unit),
            limit: "(Max ".concat(AOD_MEMORY_LIMIT_SIZE.size, " ").concat(AOD_MEMORY_LIMIT_SIZE.unit, ")"),
            hasExceeded: aod > AOD_MEMORY_LIMIT,
        },
    ];
    return memoryResults;
}
function getMessageHandler() {
    var result = '';
    var msgHandler = {
        stdOutDataHandler: function (data) {
            result = data;
        },
        stdErrorDataHandler: function (data) {
            result = data;
        },
    };
    function getResult() {
        return result;
    }
    return { msgHandler: msgHandler, getResult: getResult };
}
function spawnProcess(command, args, msgHandler) {
    return new Promise(function (resolve, reject) {
        var child = child_process.spawn(command, args);
        var procCommand = !command.includes('sign')
            ? "spawnProcess[".concat(command, " ").concat(args.join(' '), "]")
            : "spawnProcess[".concat(command, "]");
        var stdMessage = {
            stdout: '',
            stderr: '',
        };
        var encodingOption = 'utf8';
        child.stdout.setEncoding(encodingOption);
        child.stderr.setEncoding(encodingOption);
        child.stdout.on('data', function (data) {
            var _a;
            var str = data.toString();
            var lines = str.split(/(\r?\n)/g);
            console.log(lines.join(''));
            stdMessage.stdout += data;
            (_a = msgHandler === null || msgHandler === void 0 ? void 0 : msgHandler.stdOutDataHandler) === null || _a === void 0 ? void 0 : _a.call(msgHandler, str);
        });
        child.stderr.on('data', function (data) {
            var _a;
            var str = data.toString();
            console.error(str);
            stdMessage.stderr += data;
            (_a = msgHandler === null || msgHandler === void 0 ? void 0 : msgHandler.stdErrorDataHandler) === null || _a === void 0 ? void 0 : _a.call(msgHandler, str);
        });
        child.on('error', function (err) {
            console.error("".concat(procCommand, " error:").concat(err, " spawnStdMessage:").concat(stdMessage));
            reject(err);
        });
        child.on('close', function (exitCode) {
            if (exitCode !== 0) {
                console.error("".concat(procCommand, " failed with stderr:").concat(stdMessage.stderr));
                reject("".concat(procCommand, " failed with code:").concat(exitCode));
            }
            else {
                console.log("".concat(procCommand, " success"));
                resolve(stdMessage);
            }
        });
    });
}
var runMemoryFootPrint = function (mfpInputOptions) { return __awaiter(void 0, void 0, void 0, function () {
    var mfpOptions, _a, msgHandler, getResult, memoryFootPrintOptions, err_1;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                console.log('[MemoryFootPrint] mfpOptions : ' + Object.entries(mfpInputOptions).join(' '));
                mfpOptions = {
                    javaPath: mfpInputOptions.javaPath,
                    inputPath: mfpInputOptions.inputPath,
                    schemaVersion: mfpInputOptions.schemaVersion,
                };
                _a = getMessageHandler(), msgHandler = _a.msgHandler, getResult = _a.getResult;
                _b.label = 1;
            case 1:
                _b.trys.push([1, 3, , 4]);
                memoryFootPrintOptions = [
                    '-jar',
                    "".concat(MEMORY_FOOT_PRINT_TOOL_PATH),
                    '--report',
                    '--watch-face',
                    "".concat(mfpOptions.inputPath),
                    '--schema-version',
                    "".concat(mfpOptions.schemaVersion),
                    " --ambient-limit-mb",
                    "".concat(AMBIENT_MEMORY_LIMIT),
                    "--active-limit-mb",
                    "".concat(ACTIVE_MEMORY_LIMIT),
                    "--apply-v1-offload-limitations",
                    "--estimate-optimization",
                ];
                return [4 /*yield*/, spawnProcess(mfpOptions.javaPath, memoryFootPrintOptions, msgHandler)];
            case 2:
                _b.sent();
                return [2 /*return*/, memoryFootprintResultDecorator(getResult())];
            case 3:
                err_1 = _b.sent();
                console.log(err_1);
                return [2 /*return*/, undefined];
            case 4: return [2 /*return*/];
        }
    });
}); };
// Register all the above functions and objects with the relevant extension points
export function init(registerExtensions) {
    console.log("".concat(PLUGIN_NAME, " plugin is initialized"));
    registerExtensions.register('runMemoryFootPrint', 'runMemoryFootPrint', runMemoryFootPrint);
}
// Unregister all the above functions and objects with the relevant extension points
export function deinit() {
    console.log("".concat(PLUGIN_NAME, " plugin is deinitialized"));
}
